#!/bin/bash

#Decimate a datafile by a programmable factor. Remove header rows (prefixed by '#').
#Headers are removed because they would otherwise be invalid, too complex to try to fix them up.

FACTOR=10			#Default: every 10th line.
OFFSET=1			#Start printing from the 1st line, not the Nth.
INFILE=/dev/stdin		#Allow stdin if not overridden.

function show_help(){
	cat <<-EOT
		`basename $0`: subsample a file, printing only 1/N of the lines.
		Lines prefixed by '#' are ignored; printing starts at the first line.
		
		USAGE: `basename $0`  [OPTIONS]       
		OPTIONS:   -i  infile     read input from infile. [default: $INFILE].
		           -n  factor     read every nth line.    [default: $FACTOR].
		           -o  offset     start at this line number, modulo N. [default: $OFFSET]
		           -h             show this help 
		           
	EOT
	exit 0
}

while getopts hi:n:o: OPTION; do
        case "$OPTION" in
		h) show_help;;
		i) INFILE="$OPTARG";;
		n) FACTOR="$OPTARG";;
		o) OFFSET="$OPTARG";;
        esac
done

if [[ ! $FACTOR =~ ^[1-9][0-9]*$ ]]; then
	echo "Invalid factor '$FACTOR', must be a positive integer."
	exit 1
elif [[ ! $OFFSET =~ ^[0-9]*$ ]]; then
	echo "Invalid offset '$OFFSET', must be an integer."
	exit 1
fi

#check offset. If 10, set 0.


#The grep removes lines prefaced by (whitespace) + '#'.
#The perl then decimates. '$.' is an inbuilt line-counter. Factor of 1 prints every line. 
#This always prints the first line. If we want to print starting with the Nth line, change $OFFSET from 1 to 0.
grep -Ev '^[[:space:]]*#' $INFILE | perl -ne 'print if (0 == (($.- '$OFFSET') % '$FACTOR'))'

